// Copyright (c) 2005 David Drysdale.  See license.txt for license info
#include "builddate.h"
#include "gamestate.h"
#include "constants.h"
#include "globals.h"

Err LoadGameState(Boolean *pNewGame)
{
  Err err = errNone;
  DmOpenRef ref = 0;
  MemHandle h = 0;
  UInt32 romVersion;

  // Check if the 3.5 feature set is available
  FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
  got35 = (romVersion >= sysMakeROMVersion(3,5,0,sysROMStageRelease,0));

  // Assume that we find an existing game
  *pNewGame = false;

  // Attempt to get the current game state. First, see if we have a 
  // database
  ref = DmOpenDatabaseByTypeCreator('gmst', 'dmd2', dmModeReadWrite);
  if (!ref) {
    err = DmGetLastErr();
    if (err == dmErrCantFind) {
      LocalID dbID;
      UInt16 cardNo;
      LocalID appInfoID;

      // Failed to find the database -- create it instead
      err = DmCreateDatabase(0, "GameState-dmd2", 'dmd2', 'gmst', false);
      if (err) goto EXIT;
      ref = DmOpenDatabaseByTypeCreator('gmst', 'dmd2', dmModeReadWrite);
      if (!ref) {err = DmGetLastErr(); goto EXIT;}

      // Fill in the AppInfo block for the database
      err = DmOpenDatabaseInfo(ref, &dbID, NULL, NULL, &cardNo, NULL);
      if (err) goto EXIT;
      h = DmNewHandle(ref, sizeof(AppInfoType));
      if (!h) {err = dmErrMemError; goto EXIT;}
      appInfoID = MemHandleToLocalID(h);
      err = DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, 
                              NULL, NULL, NULL, NULL, &appInfoID, NULL, NULL, NULL);
      if (err != errNone) {
        MemHandleFree(h);
        goto EXIT;
      }
    } else {
      // Some failure to read the game state database other than just
      // because it doesn't exist
      goto EXIT;
    }
  }

  // If we get here, then ref is a valid database reference. Now see if 
  // we can find the (single) record with the game state
  assert(ref);
  h = DmQueryRecord(ref, 0);

  if (h) {
    BOARD *pSavedGame = (BOARD *)MemHandleLock(h);
    // We have found the record -- check it is still of the correct
    // version
    if (StrCompare(pSavedGame->buildDate, BUILDDATE) != 0) {
      // Build date in the database is different -- wipe the record
      MemHandleUnlock(h);
      DmRemoveRecord(ref, 0);
      h = 0;
    } else {
      // Build dates match, but the pointer to the structure in the database is 
      // read-only.  Hence we need to make our own in-game copy (which will get 
      // saved back into the database on exit)
      board = CreateBoard(pSavedGame->width, pSavedGame->height, pSavedGame->totalMines);
      CopyBoard(board, pSavedGame);
      MemHandleUnlock(h);
    }
  }

  if (!h) {
    // No game state record found -- attempt to create one, of a standard size
    UInt16 suggestedIndex = 0;
    h = DmNewRecord(ref, &suggestedIndex, sizeof(BOARD)+(kMediumWidth*kMediumHeight*sizeof(UInt8)));
    assert(suggestedIndex == 0);
    if (!h) {err = DmGetLastErr(); goto EXIT;}
    DmReleaseRecord(ref, 0, true);

    // Initialize the game state
    *pNewGame = true;
    board = CreateBoard(kMediumWidth, kMediumHeight, kMediumMedium);
    InitBoard(board);
  }

 EXIT:
  // Close up the database; we can open it up again for saving just before
  // exit.
  if (ref) (void) DmCloseDatabase(ref);
  return err;
}

void SaveGameState(void)
{
  Err err = errNone;
  DmOpenRef ref = 0;
  MemHandle h = 0;
  BOARD *pSavedGame;
  UInt16 suggestedIndex = 0;
  
  if (board == 0) return;

  // Write the game state back to the database
  ref = DmOpenDatabaseByTypeCreator('gmst', 'dmd2', dmModeReadWrite);
  assert(ref);

  h = DmGetRecord(ref, 0);
  assert(h);
  
  // The current game data may be a different size than the version in the database
  // Delete the old record and create a new one
  DmDeleteRecord(ref, 0);
  
  h = DmNewRecord(ref, &suggestedIndex, sizeof(BOARD)+(board->width*board->height*sizeof(UInt8)));
  assert(suggestedIndex == 0);
  if (!h) {err = DmGetLastErr(); goto EXIT;}
  
  pSavedGame = (BOARD *)MemHandleLock(h);
  (void) DmWrite(pSavedGame, 0, board, board->length);
  MemHandleUnlock(h);
  DmReleaseRecord(ref, 0, true);
  (void) DmCloseDatabase(ref);

EXIT:
  // Free off the dynamic copy of the board
  MemPtrFree(board);
}
